@@ -1,7 +1,7 @@
-__version__ = "0.10"
-__version_info__ = ['0', '10']
+__version__ = "0.10.1"
+__version_info__ = ['0', '10', '1']
 __version_vcs_info__ = {'branch': 'master',
- 'revid': '8f651db',
- 'revno': 1798,
+ 'revid': '46ed181',
+ 'revno': 1801,
  'toplevel': '/home/saxtouri/src/kamaki/kamaki'}
 __version_user_info__ = "saxtouri@terminus"
